@@ -836,6 +836,9 @@ bool GR_Win32USPGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 		RI->m_iClustSize = si.m_iLength;
 	}
 
+	// remove any justification information -- it will have to be recalculated
+	delete[] RI->m_pJustify; RI->m_pJustify = NULL;
+	
 	// to save time we will use a reasonably sized static buffer and
 	// will only allocate one on heap if the static one is too small.
 	static WCHAR wcInChars[GRWIN32USP_CHARBUFF_SIZE]; 
@@ -1073,6 +1076,11 @@ bool GR_Win32USPGraphics::shape(GR_ShapingInfo & si, GR_RenderInfo *& ri)
 		// no, this is necessary
 		RI->s_pOwnerChar = NULL;
 	}
+
+	if(RI->s_pOwnerDraw == RI)
+	{
+		RI->s_pOwnerDraw = NULL;
+	}
 	
 	return true;
 }
@@ -1158,7 +1166,7 @@ void GR_Win32USPGraphics::prepareToRenderChars(GR_RenderInfo & ri)
 	UT_uint32 iPoints = (UT_uint32)pFont->getPointSize();
 #endif
 	
-	if(iZoom == RI.m_iZoom && RI.s_pOwnerDraw == & ri && pFont->getPrintDC() != getPrintDC())
+	if(iZoom == RI.m_iZoom && RI.s_pOwnerDraw == & ri && pFont->getPrintDC() == getPrintDC())
 	{
 		// the buffer is up-to-date
 		return;
@@ -1455,7 +1463,8 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	// store the print DC used to measure the font in the font ...
 	pFont->setPrintDC(getPrintDC());
 	
-	HDC hdc = 0;
+	HDC hdc = 0; // this is the hdc to use if all fails
+	HDC hdc1 = 0; // this is the hdc to use in the first pass (null if we have script cache)
 	if(*(pFont->getScriptCache()) == NULL)
 	{
 		// need to make sure that the HDC has the correct font set
@@ -1470,10 +1479,12 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 
 		// we remember the hdc for which we measured so we can remeasure when hdc changes
 		RI.m_hdc = hdc;
+		hdc1 = hdc;
 	}
 	else
 	{
 		hdc = pFont->getPrintDC() ? pFont->getPrintDC() : m_hdc;
+		hdc1 = 0;
 		RI.m_hdc = hdc;
 	}
 	
@@ -1484,13 +1495,12 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 	if(RI.m_bShapingFailed)
 		pItem->m_si.a.eScript = GRScriptType_Undefined;
 	
-	HRESULT hRes = fScriptPlace(hdc, pFont->getScriptCache(), RI.m_pIndices,
+	HRESULT hRes = fScriptPlace(hdc1, pFont->getScriptCache(), RI.m_pIndices,
 								RI.m_iIndicesCount, RI.m_pVisAttr,
 								& pItem->m_si.a, RI.m_pAdvances, RI.m_pGoffsets, & RI.m_ABC);
 
 	if( hRes == E_PENDING)
 	{
-		UT_ASSERT_HARMLESS( hdc == 0 );
 		_setupFontOnDC(pFont, false);
 		bFontSetUpOnDC = true;
 		
@@ -1505,6 +1515,14 @@ void GR_Win32USPGraphics::measureRenderedCharWidths(GR_RenderInfo & ri)
 
 		UT_ASSERT_HARMLESS( !hRes );
 	}
+	else if(hRes)
+	{
+		// some other error, at least set the advances and offsets to 0
+		memset(RI.m_pAdvances, 0, sizeof(int) * RI.m_iIndicesCount);
+		memset(RI.m_pGoffsets, 0, sizeof(GOFFSET) * RI.m_iIndicesCount);
+		memset(&RI.m_ABC, 0, sizeof(ABC));
+	}
+	
 
 	const UT_uint32 iAdvSize = 80;
 	int iAdvances[iAdvSize];
@@ -2044,16 +2062,7 @@ UT_sint32 GR_Win32USPGraphics::countJustificationPoints(const GR_RenderInfo & ri
 	// now we need to make sense of the stats
 	if(iCountKashida)
 	{
-		RI.m_eJustification = (SCRIPT_JUSTIFY)(
-			                  SCRIPT_JUSTIFY_ARABIC_NORMAL
-                			& SCRIPT_JUSTIFY_ARABIC_KASHIDA
-			                & SCRIPT_JUSTIFY_ARABIC_ALEF
-			                & SCRIPT_JUSTIFY_ARABIC_HA
- 			                & SCRIPT_JUSTIFY_ARABIC_RA
-			                & SCRIPT_JUSTIFY_ARABIC_BA
-                			& SCRIPT_JUSTIFY_ARABIC_BARA
-			                & SCRIPT_JUSTIFY_ARABIC_SEEN
-			                & SCRIPT_JUSTIFY_BLANK );
+		RI.m_eJustification = SCRIPT_JUSTIFY_ARABIC_KASHIDA;
 		
 		return iCountSpace + iCountKashida;
 	}
@@ -2115,7 +2124,9 @@ void GR_Win32USPGraphics::justify(GR_RenderInfo & ri)
 	for(UT_uint32 i = 0; i < RI.m_iIndicesCount; ++i)
 	{
 		UT_uint32 k = pItem->m_si.a.fRTL ? RI.m_iIndicesCount - i - 1: i;
-		if(RI.m_pVisAttr[k].uJustification & RI.m_eJustification)
+		if(RI.m_pVisAttr[k].uJustification == RI.m_eJustification ||
+		   (RI.m_eJustification == SCRIPT_JUSTIFY_ARABIC_KASHIDA &&
+			RI.m_pVisAttr[k].uJustification >= SCRIPT_JUSTIFY_ARABIC_KASHIDA))
 		{
 			UT_uint32 iSpace = iExtraSpace/iPoints;
 			iExtraSpace -= iSpace;
@@ -2521,7 +2532,7 @@ GR_Win32USPFont::GR_Win32USPFont(LOGFONT & lf, double fPoints, HDC hdc, HDC prin
 	  m_printHDC(NULL),
 	  m_iScreenAscent(0)
 {
-};
+}
 
 void GR_Win32USPFont::_clearAnyCachedInfo()
 {
