@@ -428,6 +428,48 @@ bool  ABI_Collab_Export::buildPacket(const PX_ChangeRecord * pcr,
 					     PL_StruxDocHandle sdh,
 					     UT_UTF8String & sPacket)
 {
+  UT_UTF8String sType;
+  mapChangeRecordToString(pcr,sType);
+  UT_UTF8String sPos;
+  UT_UTF8String_sprintf(sPos,"%d",pcr->getPosition());
+  sPacket = "<CR=\"";
+  sPacket += sType;
+  sPacket += "\"";
+  sPacket += " pos=\"";
+  sPacket += sPos;
+  sPacket += "\"";
+  if(pcr->getType() == PX_ChangeRecord::PXT_InsertSpan)
+  {
+       const PX_ChangeRecord_SpanChange * pcrc = static_cast<const PX_ChangeRecord_SpanChange *> (pcr);
+       UT_uint32 iLen = pcrc->getLength();
+       UT_UTF8String_sprintf(sPos,"%d",iLen);
+       sPacket += " length=\"";
+       sPacket += sPos;
+       sPacket += "\"";
+
+       PT_BufIndex bi = pcrc->getBufIndex();
+       const UT_UCSChar* pChars = m_pDoc->getPointer(bi);
+
+  }
+  else if (pcr->getType() == PX_ChangeRecord::PXT_ChangeSpan)
+  {
+       const PX_ChangeRecord_SpanChange * pcrc = static_cast<const PX_ChangeRecord_SpanChange *> (pcr);
+       UT_uint32 iLen = pcrc->getLength();
+       UT_UTF8String_sprintf(sPos,"%d",iLen);
+       sPacket += " length=\"";
+       sPacket += sPos;
+       sPacket += "\"";
+  }
+  else if (pcr->getType() == PX_ChangeRecord::PXT_DeleteSpan)
+  {
+       const PX_ChangeRecord_SpanChange * pcrc = static_cast<const PX_ChangeRecord_SpanChange *> (pcr);
+       UT_uint32 iLen = pcrc->getLength();
+       UT_UTF8String_sprintf(sPos,"%d",iLen);
+       sPacket += " length=\"";
+       sPacket += sPos;
+       sPacket += "\"";
+  }
+  sPacket += ">";
   return true;
 }
 
