@@ -26,12 +26,15 @@
 class GR_Abi_MathGraphicDevice : public MathGraphicDevice
 {
 protected:
-  GR_Abi_MathGraphicDevice(const SmartPtr<class AbstractLogger>&, class GR_Graphics*);
+  GR_Abi_MathGraphicDevice(const SmartPtr<class AbstractLogger>&,
+			   const SmartPtr<class Configuration>&,
+			   class GR_Graphics*);
   virtual ~GR_Abi_MathGraphicDevice();
 
 public:
-  static SmartPtr<GR_Abi_MathGraphicDevice> create(const SmartPtr<class AbstractLogger>& pLogger, class GR_Graphics* pGr)
-  { return new GR_Abi_MathGraphicDevice(pLogger, pGr); }
+  static SmartPtr<GR_Abi_MathGraphicDevice> create(const SmartPtr<class AbstractLogger>& pLogger,
+						   const SmartPtr<class Configuration>&,
+						   class GR_Graphics* pGr);
 
 private:
   SmartPtr<class GR_Abi_AreaFactory> m_abiFactory;
