@@ -96,6 +96,8 @@ public:
   */
   const XML_Char ** getAbiPageAtts(const XML_Char * masterName)
   {
+    UT_return_val_if_fail(masterName != NULL, (const XML_Char **)m_pageAtts);
+
     if (UT_strcmp (m_name.c_str(), masterName))
       // FIXME can there be more than one master-page?
       UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
@@ -612,6 +614,8 @@ const XML_Char* IE_Imp_OpenWriter::mapStyle (const XML_Char * name) const
 
 const OO_Style * IE_Imp_OpenWriter::mapStyleObj (const XML_Char * name) const
 {
+  if (!name)
+    return NULL;
   return m_styleBucket.pick((const char *)name);
 }
 
@@ -622,7 +626,7 @@ const OO_Style * IE_Imp_OpenWriter::mapStyleObj (const XML_Char * name) const
  * Create a new OpenWriter importer object
  */
 IE_Imp_OpenWriter::IE_Imp_OpenWriter (PD_Document * pDocument)
-  : IE_Imp (pDocument), m_oo (0), m_pSSListener(0)
+  : IE_Imp (pDocument), m_oo (0), m_pSSListener(0), m_bOpenDocument(false)
 {
 }
 
@@ -860,28 +864,23 @@ UT_Error IE_Imp_OpenWriter::_handleMimetype ()
 
   GsfInput * input = gsf_infile_child_by_name(m_oo, "mimetype");
 
-  if (!input)
-    return UT_ERROR;
+  m_bOpenDocument = FALSE;
 
-  UT_UTF8String mimetype = "";
+  if (!input) {
+    UT_DEBUGMSG(("Error: didn't get a mimetype. Assuming that it's a application/vnd.sun.xml.writer document\n"));
+    UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+    return UT_OK;
+  }
+
+  UT_UTF8String mimetype;
   if (gsf_input_size (input) > 0) {
-    while ((len = gsf_input_remaining (input)) > 0) {
-      // FIXME: we want to pass the stream in chunks, but libXML2 finds this disagreeable.
-      // we probably need to pass some magic to our XML parser? 
-      // len = UT_MIN (len, BUF_SZ);
-      if (NULL == (data = gsf_input_read (input, len, NULL))) {
-        g_object_unref (G_OBJECT (input));
-        return UT_ERROR;
-      }
-      mimetype += (const char*)data;
-	}
+    mimetype.append((const char *)gsf_input_read(input, gsf_input_size (input), NULL), gsf_input_size (input));
   }
   
   if (!UT_strcmp("application/vnd.oasis.opendocument.text", mimetype.utf8_str())) {
     m_bOpenDocument = TRUE;
   }
   else if (!UT_strcmp("application/vnd.sun.xml.writer", mimetype.utf8_str())) {
-    m_bOpenDocument = FALSE;
   }
   else {
 	UT_DEBUGMSG(("*** Unknown mimetype '%s'\n", mimetype.utf8_str()));  
@@ -960,6 +959,16 @@ public:
   
   virtual ~OpenWriter_StylesStream_Listener ()
   {
+    m_styleNameMap.purgeData();
+  }
+
+  UT_UTF8String getStyleName(const UT_UTF8String & in) const
+  {
+    UT_UTF8String * name = m_styleNameMap.pick(in.utf8_str());
+    if (!name)
+      return UT_UTF8String(in);
+    else
+      return *name;
   }
   
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
@@ -980,15 +989,29 @@ public:
       if (attr)
 	m_name = attr;
 
+      attr = UT_getAttribute("style:display-name",atts);
+      if (attr)
+	m_displayName = attr;
+
       if (m_name != "Standard") {
 	attr = UT_getAttribute("style:parent-style-name",atts);
-	if (attr)
-	  m_parent = attr;
+	if (attr) {
+	  if (!UT_strcmp(attr, "Standard")) {
+	    m_parent = "Normal";
+	  } else {
+	    m_parent = attr;
+	  }
+	}
 	
 	attr = UT_getAttribute("style:next-style-name",atts);
-	if (attr)
-	  m_next = attr;
-	
+	if (attr) {
+	  if (!UT_strcmp(attr, "Standard")) {
+	    m_next = "Normal";
+	  } else {
+	    m_next = attr;
+	  }
+	}
+
 	attr = UT_getAttribute("style:family",atts);
 	if (attr && strcmp (attr, "paragraph") != 0)
 	  m_type = CHARACTER;
@@ -1002,25 +1025,25 @@ public:
 	m_type = PARAGRAPH;
       }
       DELETEP(m_ooStyle);
-	  m_ooStyle = NULL;
+      m_ooStyle = NULL;
     }
     else if (/* SXW || ODT */
-			(!strcmp (name, "style:properties") || !strcmp (name, "style:page-layout-properties")) && 
-			m_pageMaster) {
+	     (!strcmp (name, "style:properties") || !strcmp (name, "style:page-layout-properties")) && 
+	     m_pageMaster) {
       // page setup, because we are in page-master subtree
       m_ooPageStyle.appendPageMaster(m_pageMaster, atts);
     }
     else if (!strcmp (name, "style:properties") || 				/* SXW */
-			 !strcmp (name, "style:text-properties") || 		/* ODT */
-			 !strcmp (name, "style:paragraph-properties")) {
-
-		if (m_ooStyle == NULL) {
-	      getDocument()->getStyle (m_parent.utf8_str(), &m_pParentStyle);
-    	  m_ooStyle = new OO_Style (atts, m_pParentStyle, m_bOpenDocument);
-		}
-		else {
-		  m_ooStyle->parse(atts);
-		}
+	     !strcmp (name, "style:text-properties") || 		/* ODT */
+	     !strcmp (name, "style:paragraph-properties")) {
+      
+      if (m_ooStyle == NULL) {
+	getDocument()->getStyle (m_parent.utf8_str(), &m_pParentStyle);
+	m_ooStyle = new OO_Style (atts, m_pParentStyle, m_bOpenDocument);
+      }
+      else {
+	m_ooStyle->parse(atts);
+      }
     }
   } 
 
@@ -1036,8 +1059,16 @@ public:
 	const XML_Char * atts[11];
 	atts[propCtr++] = "type";
 	atts[propCtr++] = (m_type == PARAGRAPH ? "P" : "C");
+
 	atts[propCtr++] = "name";
-	atts[propCtr++] = m_name.utf8_str();
+	if (m_displayName.size()) {	 
+	  atts[propCtr++] = m_displayName.utf8_str();
+	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_displayName));
+	}
+	else {
+	  atts[propCtr++] = m_name.utf8_str();
+	  m_styleNameMap.insert (m_name.utf8_str(), new UT_UTF8String(m_name));
+	}
 
 	if (m_ooStyle) {
 	  atts[propCtr++] = "props";
@@ -1060,10 +1091,11 @@ public:
       }
 
       m_name.clear ();
+      m_displayName.clear ();
       m_parent.clear ();
       m_next.clear ();
       DELETEP(m_ooStyle);
-	  m_ooStyle = NULL;
+      m_ooStyle = NULL;
     }
   }
   
@@ -1084,6 +1116,7 @@ public:
 private:
 
   UT_UTF8String m_name;
+  UT_UTF8String m_displayName;
   UT_UTF8String m_parent;
   UT_UTF8String m_next;
   enum { CHARACTER, PARAGRAPH } m_type;
@@ -1092,6 +1125,8 @@ private:
   OO_PageStyle m_ooPageStyle;
   const XML_Char * m_pageMaster;
   const bool m_bOpenDocument;
+
+  UT_GenericStringMap<UT_UTF8String *> m_styleNameMap;
 };
 
 /*!
@@ -1114,7 +1149,8 @@ class ABI_EXPORT OpenWriter_ContentStream_Listener : public OpenWriter_Stream_Li
 private:
   UT_UCS4String m_charData;
   bool m_bAcceptingText;
-  bool m_bInSection;  
+  bool m_bInSection;
+  bool m_bInTOC;
 
   UT_UTF8String m_curStyleName;
 
@@ -1132,7 +1168,7 @@ private:
 
 public:
   OpenWriter_ContentStream_Listener ( IE_Imp_OpenWriter * importer, OpenWriter_StylesStream_Listener * pSSListener, bool bOpenDocument )
-    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), 
+    : OpenWriter_Stream_Listener ( importer ), m_bAcceptingText(false), m_bInSection(false), m_bInTOC(false),
       m_pSSListener(pSSListener), m_imgCnt(0), m_row(0), m_col(0), m_cel(0), m_bOpenDocument(bOpenDocument)
   {
   }
@@ -1143,6 +1179,10 @@ public:
 
   virtual void startElement (const XML_Char * name, const XML_Char ** atts) 
   {
+    // if we're inside a TOC, ignore the contents since SXW/ODT include a copy of the TOC
+    if (m_bInTOC)
+      return;
+
     if (!UT_strcmp(name, "office:body"))
       {
       }
@@ -1151,23 +1191,17 @@ public:
 	const XML_Char * oo_sty = UT_getAttribute ("text:style-name", atts);
 	const XML_Char * abi_sty = _mapStyle(oo_sty);
 
-	_appendSection(abi_sty);
-	m_bInSection = true;
+	_insureInSection(abi_sty);
       }
     else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
-        if (!m_bInSection)
-	  {
-	      _appendSection(NULL);
-	      m_bInSection = true;
-	  }
-	const XML_Char * oo_sty = UT_getAttribute ("text:style-name", atts);
-	const OO_Style * abi_sty = _mapStyleObj (oo_sty);
+	UT_UTF8String oo_sty;
+	const OO_Style * abi_sty = _mapStyleObj (UT_getAttribute ("text:style-name", atts), oo_sty);
 
 	if (abi_sty) {
 	  // append an empty paragraph with this one char
 	  if (abi_sty->getColBreakBefore () || abi_sty->getPageBreakBefore ()) {
-	    getDocument()->appendStrux(PTX_Block, NULL);
+	    _insureInBlock(NULL);
 	    UT_UCSChar ucs = (abi_sty->getColBreakBefore () ? UCS_VTAB : UCS_FF);
 	    getDocument()->appendSpan (&ucs, 1);
 	  }
@@ -1177,18 +1211,18 @@ public:
 	  props[1] = abi_sty->getAbiStyle ();
 	  props[2] = 0;
 	  
-	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
+	  _insureInBlock((const XML_Char **)props);
 	}
-	else if (oo_sty) {
+	else if (oo_sty.size()) {
 	  const XML_Char * props[3];
 	  props[0] = "style";
-	  props[1] = oo_sty;
+	  props[1] = oo_sty.utf8_str();
 	  props[2] = 0;
 
-	  getDocument()->appendStrux(PTX_Block, (const XML_Char **)props);
+	  _insureInBlock((const XML_Char **)props);
 	}
 	else
-	  getDocument()->appendStrux(PTX_Block, NULL);
+	  _insureInBlock(NULL);
 	m_bAcceptingText = true;
       }
     else if (!UT_strcmp(name, "text:span"))
@@ -1211,6 +1245,28 @@ public:
 	m_charData += UCS_LF;
 	_flush ();
       }
+    else if (!UT_strcmp(name, "text:ordered-list") || !UT_strcmp(name, "text:unordered-list"))
+      {
+	const XML_Char * list_atts[15];
+	UT_uint32 iOffset = 0;
+
+	// list type
+	list_atts[iOffset++] = "type";
+	if (!UT_strcmp(name, "text:ordered-list"))
+	  list_atts[iOffset++] = "Numbered List";
+	else
+	  list_atts[iOffset++] = "Bullet List";
+
+	list_atts[iOffset++] = "id";
+	list_atts[iOffset++] = "0";
+
+	// NULL
+	list_atts[iOffset++] = 0;  
+
+	getDocument()->appendList(list_atts);
+
+	UT_DEBUGMSG(("DOM: appended a list\n"));
+      }
     else if (!UT_strcmp(name, "style:style"))
       {
 	m_curStyleName.clear ();
@@ -1239,18 +1295,25 @@ public:
       _insertBookmark (name, "start");
       _insertBookmark (name, "end");
     }
+    else if (!UT_strcmp(name, "text:table-of-content"))
+    {
+      _flush ();
+      _insureInBlock(NULL);
+      
+      getDocument()->appendStrux(PTX_SectionTOC, NULL);
+      getDocument()->appendStrux(PTX_EndTOC, NULL);
+
+      m_bInTOC = true;
+    }
     else if (!UT_strcmp(name, "draw:image"))
       {
 	_flush ();
+	_insureInBlock(NULL);
 	_insertImage (atts);
       }
     else if (!UT_strcmp(name, "table:table"))
       {
-        if (!m_bInSection)
-	  {
-	      _appendSection(NULL);
-	      m_bInSection = true;
-	  }
+	_insureInSection(NULL);
 	_openTable (atts);
       }
     else if (!UT_strcmp(name, "table:table-column"))
@@ -1328,7 +1391,7 @@ public:
     else if (!UT_strcmp(name, "text:p" ) || !UT_strcmp(name, "text:h" ))
       {
 	_flush ();
-	m_bAcceptingText= false;
+	m_bAcceptingText = false;
       }
     else if (!UT_strcmp(name, "text:span"))
       {
@@ -1336,11 +1399,17 @@ public:
 	_popInlineFmt();
 	getDocument()->appendFmt(&m_vecInlineFmt);
       }
+    else if (!UT_strcmp(name, "text:ordered-list") || !UT_strcmp(name, "text:unordered-list"))
+      {
+      }
     else if (!UT_strcmp(name, "text:a"))
       {
 	_flush ();
 	getDocument()->appendObject(PTO_Hyperlink, NULL);
       }
+    else if (!UT_strcmp(name, "text:table-of-content")) {
+      m_bInTOC = false;
+    }
     else if (!UT_strcmp(name, "table:table"))
       {
 	_closeTable ();
@@ -1378,7 +1447,7 @@ public:
 
   virtual void charData (const XML_Char * buffer, int length)
   {
-    if (buffer && length && m_bAcceptingText)
+    if (buffer && length && m_bAcceptingText && !m_bInTOC)
       m_charData += UT_UCS4String (buffer, length, true);
   }
 
@@ -1465,7 +1534,7 @@ private:
 	FREEP(mimetype);
 	goto Cleanup;
       }
-    
+
     if (!getDocument()->createDataItem(propsName.c_str(), false,
 				       pictData, (void*)mimetype, NULL))
       {
@@ -1476,8 +1545,24 @@ private:
     DELETEP(importer);
   }
 
-  void _appendSection(const XML_Char * props) 
+  void _insureInBlock(const XML_Char ** atts)
   {
+    if (m_bAcceptingText)
+      return;
+
+    _insureInSection(NULL);
+
+    if (!m_bAcceptingText) {
+      getDocument()->appendStrux(PTX_Block, (const XML_Char**)atts);	
+      m_bAcceptingText = true;
+    }
+  }
+
+  void _insureInSection(const XML_Char * props) 
+  {
+    if (m_bInSection)
+      return;
+
     UT_String allProps(props);
     allProps += m_pSSListener->getSectionProps();
 
@@ -1486,6 +1571,9 @@ private:
     atts[1] = allProps.c_str();
     atts[2] = 0;
     getDocument()->appendStrux(PTX_Section, (const XML_Char**)atts);	
+
+    m_bInSection = true;
+    m_bAcceptingText = false;
   }
   
   void _insertBookmark (const XML_Char * name, const XML_Char * type)
@@ -1542,6 +1630,8 @@ private:
   void _closeTable ()
   {
     getDocument()->appendStrux(PTX_EndTable, NULL);
+
+    m_row = m_cel = m_col = 0;
   }
 
   void _closeColumn ()
@@ -1566,12 +1656,14 @@ private:
 
   const XML_Char* _mapStyle (const XML_Char * name) const
   {
-    return getImporter ()->mapStyle(name);
+    UT_UTF8String styleName = m_pSSListener->getStyleName(name);
+    return getImporter ()->mapStyle(styleName.utf8_str());
   }
 
-  const OO_Style * _mapStyleObj (const XML_Char * name) const
+  const OO_Style * _mapStyleObj (const XML_Char * name, UT_UTF8String & oo_sty) const
   {
-    return getImporter ()->mapStyleObj (name);
+    oo_sty = m_pSSListener->getStyleName(name);
+    return getImporter ()->mapStyleObj (oo_sty.utf8_str());
   }
 
   bool _pushInlineFmt(const XML_Char ** atts)
